;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xc5fcd8f1b87744c7e5fa1f464c7ad2f4 . "./code.cg2js/cg-acache.fasl")
(#x734c9449fb2ed6eed470890838f1f641 . "./code.cg2js/cg-base.fasl")
(#x1a56b7731d53fc0e528fe6ba449313a0 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xf2cdd1842bbbedbcca9bd1dc1596c026 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xa13a402c575cf1284ec9574907899f33 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x0ce8d1ad68bd647ee5d468fa06396973 . "./code.cg2js/cg-button.fasl")
(#x3772bf5aa14b61111a324e61e0640a2b . "./code.cg2js/cg-calendar.fasl")
(#xc50df7fb8de44404f664ecf5ca4c46ce . "./code.cg2js/cg-caret.fasl")
(#xd881387f2b53e3ccc0b82641fe1be844 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xc6c22a8cc45a61ae5d5191a738efa3fb . "./code.cg2js/cg-chart-widget.fasl")
(#x964fb54294860c3f661670d9e358b4e6 . "./code.cg2js/cg-check-box.fasl")
(#x01b8ecad6800760af2a02ea994135395 . "./code.cg2js/cg-choice-list.fasl")
(#x26ff415adf9ca94c67ad07d0439dc10f . "./code.cg2js/cg-choose-printer.fasl")
(#xdae5be48d390db07c29108acacc07e43 . "./code.cg2js/cg-class-grid.fasl")
(#x17406116e03d8ac2aee864f4df6e8972 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x6730195cfdd32c9c37010c2f95a1b0d3 . "./code.cg2js/cg-class-support.fasl")
(#x11185edc432418dc3ed6d35e89651177 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x5d92a9de072cd99cbf5259058bd3d037 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x2fb2072304af1733b6a4d7c7eb7ff43e . "./code.cg2js/cg-clipboard.fasl")
(#x7ca9d99abd91d079e9f4489452c157c0 . "./code.cg2js/cg-color-dialog.fasl")
(#xeb69f5a866329451e17542d429071cb9 . "./code.cg2js/cg-combo-box.fasl")
(#x5de1e46ad855f8a6565829191a63c960 . "./code.cg2js/cg-common-control.fasl")
(#x8eff91b79af271e74475f8beb486da06 . "./code.cg2js/cg-comtab.fasl")
(#x7304b7f86c0cd866deca2efb04ac6452 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x4470a8cee15a8fa41fa86bafc8024925 . "./code.cg2js/cg-curve.fasl")
(#xac15b4dfec53161a08ccfd4db738e1ae . "./code.cg2js/cg-dde-utils.fasl")
(#x27c67dee244537a5a6660f940b6bd6d0 . "./code.cg2js/cg-dev.fasl")
(#x744dd1cbf69876271d6921934fcb2393 . "./code.cg2js/cg-dialog-item.fasl")
(#xe2f2450c2926e2799950eb0f53909b2a . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x2b4398f74dcb72c84bf0827f16cf06e6 . "./code.cg2js/cg-directory-dialog.fasl")
(#x3a63b1132059e12224a00776d216f231 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x3c41c2d010c0840f6c4f53df508f85b8 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x858ddd5560c15df57a3051660a5a8bdf . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x13bda3d646297f5650aeb404a3d27cba . "./code.cg2js/cg-drawable.fasl")
(#x47b60ced7ea1d1fdc5a574903187cb2c . "./code.cg2js/cg-dropping-outline.fasl")
(#xcbd8b19f6d0124b800df85ce36a59db8 . "./code.cg2js/cg-edit-in-place.fasl")
(#x48cee6caf53954c0afd4d06d0dc9e69e . "./code.cg2js/cg-editable-text.fasl")
(#xce2f654f455655d7d1f9ac7ebc6ec4f8 . "./code.cg2js/cg-file-dialog.fasl")
(#x6d004ec66d210feff6e2a3f101ff4fa8 . "./code.cg2js/cg-fill-texture.fasl")
(#xff300386c7e17edd22607ef6a063eb66 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x189365bf21ccf85f3a58fa5bcecf4566 . "./code.cg2js/cg-font-dialog.fasl")
(#x5e32221ff1ac2c9a8dd3deec8e36ac26 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x46d9cc1bfb7f9fef3f3c35cf14d4ef64 . "./code.cg2js/cg-get-pixmap.fasl")
(#x237fd6bffe99d15d8da7e8100592f484 . "./code.cg2js/cg-get-position.fasl")
(#xc7b29aa10a6cadef72dbbb7fc9fce903 . "./code.cg2js/cg-graphics-context.fasl")
(#xc534348dcf215e729d6a60b4e9cfde84 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x57ab7d17d167eee73994b33ac7f9efa3 . "./code.cg2js/cg-grid-widget.fasl")
(#x468fb598baa52d3053a39c731095d08a . "./code.cg2js/cg-group-box.fasl")
(#x5c27f454738581ce3b97b72ffe166684 . "./code.cg2js/cg-header-control.fasl")
(#x0540ab801640b73d823074ddee18cbc4 . "./code.cg2js/cg-hotspot.fasl")
(#x39ed105936bd2aceda0f9fde0fedac27 . "./code.cg2js/cg-html-dialog.fasl")
(#x27c8701541a3a35f510c47b8e8a67fe4 . "./code.cg2js/cg-html-widget.fasl")
(#xf5dca23b4339fe7c53d95f83a7cd7b57 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x60bc74ff61eb4ce8db902e4f48ad60e5 . "./code.cg2js/cg-icon.fasl")
(#x8e0d2c66429104ef88bc50769edcf7f8 . "./code.cg2js/cg-ie.fasl")
(#xe9d893515ffb61fba450d7d13ed03a4b . "./code.cg2js/cg-intersect-posbox.fasl")
(#x1d2c32979ad2563058cb76b4a6547e13 . "./code.cg2js/cg-intersect.fasl")
(#x5bc914462d4f0265dca6be6c630edc1d . "./code.cg2js/cg-item-list.fasl")
(#x1bfd3ad107ac6a12cefd1b0a2364f712 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xb0003774855a874a0bb61b9e23943b95 . "./code.cg2js/cg-lamp.fasl")
(#xd9a255b92d735305d9085da1b85144bc . "./code.cg2js/cg-layout.fasl")
(#xbf2c156a804213a4d2f61f6551f2c386 . "./code.cg2js/cg-lettered-menu.fasl")
(#x7e491706962274ff0f9d3f5aae6deab9 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xe24a30664fc21c4b524e8de77250d035 . "./code.cg2js/cg-lisp-text.fasl")
(#xfeb38828ce7557975294a90d9938c283 . "./code.cg2js/cg-lisp-widget.fasl")
(#x20453db07d0cec55b78d68cd4cb26cdf . "./code.cg2js/cg-list-view.fasl")
(#xc86ef7a67beb2243964d65bdddee5205 . "./code.cg2js/cg-mci.fasl")
(#xe57711c8d60aeb78c32d2344fe359ce5 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x139e58716a9ea829ae18e0926c6ddfb2 . "./code.cg2js/cg-menu.fasl")
(#xb77bc06c3b47074ccb6772835e236a76 . "./code.cg2js/cg-message-dialog.fasl")
(#x8a48586e1ab427bea25b1e2de16ec146 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xbab389e38bbc6312a8aecbcaf575ea61 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xb3b27ee3aea104327800d5da56e3d5d4 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x03f0a22b1d1695d1afc90ccb8b20a914 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x4d262f66068d65033bcad18e8c9a276a . "./code.cg2js/cg-multi-picture-button.fasl")
(#xc0a569835105be05df2b0f7748482f29 . "./code.cg2js/cg-nodes.fasl")
(#x1fafefd73fe76af33897707e147937ba . "./code.cg2js/cg-object-editor-layout.fasl")
(#xb4d6333e6507888f25baf106fb50ce5f . "./code.cg2js/cg-object-editor.fasl")
(#x1d27d46c0dbd4c8f679c9bba168c23de . "./code.cg2js/cg-ocx.fasl")
(#xc906641274bce6656c48bcf280dc5923 . "./code.cg2js/cg-os-widget.fasl")
(#xb13eb44715a542dc3dc3ab9302b9502a . "./code.cg2js/cg-os-window.fasl")
(#x6dfa8b5c8688b246fac77a783f48ed50 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xe694d9e7bf42d94c61433628c3587ee0 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1e7e359a0c4c4bfa8efca766880447d2 . "./code.cg2js/cg-outline.fasl")
(#x6b8ffbde2c7af5d24484a316c326a1a4 . "./code.cg2js/cg-palette.fasl")
(#xeccf1ac0684554496189bf7b2a6f1b92 . "./code.cg2js/cg-paren-matching.fasl")
(#x63e9fcf2233835613851e714f312df94 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x74999e3771c7d0ae22810b9aa90decc3 . "./code.cg2js/cg-picture-widget.fasl")
(#x10315df44ac87c1434ce88f5e4865244 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x148d18303e32444a0bf20b1d1f4ef2b7 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x6341b9d0193dc64ce20e912f117315e6 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xf1f7c1be48fcc4a6aced20007e00c99f . "./code.cg2js/cg-pixmap-widget.fasl")
(#x55db2322b95f30429606bc80c522f9c4 . "./code.cg2js/cg-pixmap.fasl")
(#x0c9991c9677806bb26df673cc2e62e6f . "./code.cg2js/cg-player.fasl")
(#xf4ab271735fc6c573557839ec918125f . "./code.cg2js/cg-plot-widget.fasl")
(#x03b51cf131516deb4b91df0a529affb4 . "./code.cg2js/cg-printing.fasl")
(#x0f58ce4d705c0e618f14916a71e11eed . "./code.cg2js/cg-progress-indicator.fasl")
(#xfb0d8e0ebc4e121bc36bc9bf090145cb . "./code.cg2js/cg-project-window.fasl")
(#x3f12fc2291d2f63bf1ccc910d33959e2 . "./code.cg2js/cg-property.fasl")
(#xe86839eec86814bab7b2260d5064e043 . "./code.cg2js/cg-radio-button.fasl")
(#x4c02e631c29dbe080f60f226fd88f916 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xd263a7030dba10b0d053509967802838 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xa12197e2848944c7c6d6e3934334c1ef . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x599d54737d021e7fb9f6059a10cfdf72 . "./code.cg2js/cg-rich-edit.fasl")
(#x42973ce3a1adffbddc32ef79fa368bcd . "./code.cg2js/cg-sample-file-menu.fasl")
(#x9fd18f739ea62544b7b4c5ff597d03b5 . "./code.cg2js/cg-scaling-stream.fasl")
(#x74426e400c5ddae5fe5dedbae9fc4969 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x154885b254a40b13113d9267bfd368a2 . "./code.cg2js/cg-scroll-bar.fasl")
(#xf7d05ebbd6134be916deab54ed948154 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x5430a51888aa854e568381fff7ebd08b . "./code.cg2js/cg-selected-object.fasl")
(#x96ea2332590168c12a433deeb1a1afb7 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xd3009bbc79d0b45f7dd8cc3c04784db3 . "./code.cg2js/cg-split-bar.fasl")
(#xd078cbfebb8fa8c1d5f7968d4775d4f0 . "./code.cg2js/cg-static-text.fasl")
(#x697e7137d7837eb725b2ce3d2ea1639c . "./code.cg2js/cg-status-bar.fasl")
(#xeddc4d99e8e324b0a4a6840b120ec3a4 . "./code.cg2js/cg-string-dialog.fasl")
(#x842f637ddf6542c0acb56c2772ed00e9 . "./code.cg2js/cg-tab-control.fasl")
(#x5d5943aaabb3e635db5327e42c49d53b . "./code.cg2js/cg-template-string.fasl")
(#xb7fa29ae1613ab37b014b8164312896d . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x70b6e7cd15c49d8a18a47c2da2b6b5b6 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xea66cc9adc9b64d345e691c48832001f . "./code.cg2js/cg-text-edit-pane.fasl")
(#x172aeab6f009fec7a22e8b9be1d564cf . "./code.cg2js/cg-text-or-combo.fasl")
(#x7ac8b8cbebf969a5201af9ed8cae4016 . "./code.cg2js/cg-text-widget.fasl")
(#x6667ef57686ffca575b50fef26774cde . "./code.cg2js/cg-timer.fasl")
(#x3932eeb1deafe624ba5dcadaa1fc5ec2 . "./code.cg2js/cg-toggling-widget.fasl")
(#x598d4b2829e669efd3aeaf04b193a6ac . "./code.cg2js/cg-toolbar.fasl")
(#xea70701aa53c5b234aa071bceb69fe56 . "./code.cg2js/cg-tooltip.fasl")
(#xf8df69010f47f914f779c06938c70502 . "./code.cg2js/cg-touch.fasl")
(#x8cc2121230e485145e22683cdf3a0ff6 . "./code.cg2js/cg-trackbar.fasl")
(#x8fe86f4a09c5b2ef3f40ee49dd3c3c8e . "./code.cg2js/cg-tray.fasl")
(#x59e8fa750124612e6d68058d9539b1c5 . "./code.cg2js/cg-up-down-control.fasl")
(#x78cc9b37c7ed6f9ccf152998e72390a5 . "./code.cg2js/cg-utility-dialog.fasl")
(#x8e717fa50c7f776bfebdb7925659b8e6 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x86fb7d58c858ede80df35e6b6841aa78 . "./code.cg2js/cg-web-browser.fasl")
(#x9a2433daba3ee003ebf7de2d6e6e38b4 . "./code.cg2js/cg-wrap-string.fasl")
(#x0fb518245afec2f5f70019625d9bb30e . "./code.cg2js/cg-yes-no-list.fasl")
(#x4a6fb445aa2629290cabb78abe173db1 . "./code.cg2js/cg.fasl")
(#xbd93d6f1e952267fa0db530e424b100f . "./code.cg2js/cg-yes-no-string.fasl")
(#x23cbedab51037bc68d32046e6a7142d2 . "./code.idejs/ide-acache.fasl")
(#x63f471d5774d478e9a835cc907e04a51 . "./code.idejs/ide-apropos.fasl")
(#xbf0aa436db7fb45a089dae37cecc934d . "./code.idejs/ide-base.fasl")
(#x7d67bf82405e9c968acfb51af1386b3a . "./code.idejs/ide-class-browser.fasl")
(#x99dcddc782b952b4c21cb83f775dcf2d . "./code.idejs/ide-class-grapher.fasl")
(#x77cfa47d850542ab7a3fdfda22e30585 . "./code.idejs/ide-clipboard.fasl")
(#x119720ffd791b97a4413dde04f9c8172 . "./code.idejs/ide-dev.fasl")
(#x4f4a4b043da0aa8e4a5a5367b905b3ad . "./code.idejs/ide-editor.fasl")
(#xdea2d4e97a96dc7489c3bb89dabe980e . "./code.idejs/ide-find-definitions.fasl")
(#x1f473600e9e1a527b4ff7f099aad34b7 . "./code.idejs/ide-find-in-files.fasl")
(#x1b5647c4e93183ccccb1ccc86c55edb4 . "./code.idejs/ide-form.fasl")
(#x97b87f780802ebca50ca0b6fa4760d59 . "./code.idejs/ide-grapher.fasl")
(#x5e3a6ace41aebe14478d00702d2efcee . "./code.idejs/ide-inspector.fasl")
(#x22a1aaa42ce0bea346b57f9cd9f36b97 . "./code.idejs/ide-intro.fasl")
(#x501fb3a4a59a6a86e327d39964342176 . "./code.idejs/ide-keyshort.fasl")
(#x476df8c283da917d662430f908142cb2 . "./code.idejs/ide-navigator.fasl")
(#xbfb4cf4b139b160c7db349b415937d6f . "./code.idejs/ide-package-list.fasl")
(#x7ebfe2fc14317939fa12ec5b4062c448 . "./code.idejs/ide-patch-dialog.fasl")
(#xb6b9777aabc55fb7f886307d90f046fc . "./code.idejs/ide-process-browser.fasl")
(#xbfa2f41284d6f9a4c9dde1682121f348 . "./code.idejs/ide-project.fasl")
(#x66b18e01db43666584c81c97f6da720b . "./code.idejs/ide-trace.fasl")
(#x3f5d6d528449188e472270f4bcccc6e1 . "./code.idejs/ide-window-list.fasl")
(#x1330dbfbffe7db5f2a3f7202389b1de8 . "./code.idejs/ide.fasl")
(#xf2773f648685397c1e60fb942db24e35 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
